<?php
/**
 * 未完了ジョブの通知を行います。
 * 設定ファイルを読み込み、そこで指定された期限を超えたジョブを探し
 * 担当者に定型メッセージを送信します。
 * 担当者が未定であればアクティブユーザ全員に投げます。
 */

define("LOG_DEFAULT", LOG_LOCAL4);
define("MAIL_TMPL_DIR", "../etc/");
define("MAIL_TMPL", "NoticeMail.txt");
define("DEFAULT_MAIL_FROM" , "noreply@localhost.localdomain");
define("DEFAULT_SUBJECT" , "Notification from CuMAS");
include_once '../lib/cumas_config.php';
include_once '../lib/cumas_smarty.php';

try {
    $config = new CuMAS_Config();
} catch (CuMAS_Exception $e) {
    $e->log(LOG_DEFAULT, __FILE__);
    exit(1);
}
$logFacility = DgCommon_set_logfacility($config->syslogfascility);


try {
    $dsn = "pgsql:host=$config->dbserver;"
         . "port=$config->dbport;dbname=$config->dbname";

    $pdo = new PDO($dsn, $config->dbuser, $config->dbpasswd);
    $pdo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);

    /*
     * 完了ステータス条件を作る
     * WHERE (co_status = 1 or co_status = 2)
     * の（）の中身みたいの。
     * incompleteは数字の配列であることが読み込みチェックで保証されている
     * また、numericな文字列の配列となっている。
     */
    $sql_incomplete = implode(
        " or ",
        array_map(function ($s) { return "co_status = $s"; },
                  $config->incomplete)
    );

    $sql = "SELECT co_id, co_inquiry, co_lastupdate,"
         . " co_limit, co_child_no, co_parent,"
         . " us_name, us_mail, us_active, st_status"
         . " ma_from_addr, ma_subject"
         . " FROM contact_tab"
         . " LEFT OUTER JOIN user_tab ON co_us_id = us_id"
         . " JOIN status_tab ON co_status = st_id"
         . " JOIN mail_tab ON co_ma_id = ma_id"
         . " WHERE"
         . " co_lastupdate < (current_timestamp + '-{$config->latedays} days')"
         . " AND ({$sql_incomplete})";
    /*
     * 可変部分は数字であることが保証されている
     * (ので、エスケープの必要なし)
     */
    $stmt = $pdo->query($sql, PDO::FETCH_ASSOC);
    // もし0件だったら(その時はfalse)何もせず終了
    if (!$stmt) {
        exit(0);
    }

    /* メールテンプレート指定 */
    $mailTmpl = new CuMAS_Smarty(["escape_html" => false,
                                  "template_dir" => MAIL_TMPL_DIR]);
    $tags["latedays"] = $config->latedays;
    $allUsers = "";	/* 担当者なし時の送信先作成に使う */

    /* mb_send_mail用設定 */
    mb_language("japanese");
    mb_internal_encoding("UTF-8");

    foreach ($stmt as $row) {
        /* URLに使うジョブ番号をセット */
        $tags["co_id"] = $row["co_id"];

        /* 表示するお問い合わせ番号 */
        if ($row["co_child_no"]) {
            $tags["contact_no"] = "{$row['co_parent']}.{$row['co_child_no']}";
        } else {
            $tags["contact_no"] = $row["co_id"];
        }

        /* mb_send_mailに渡すTo引数をセット */
        // 担当者がいて、かつアクティブ
        // いなければnull、非アクティブならfalse
        if ($row["us_active"]) {
            $to = $row["us_mail"];
            $tags["user"] = $row["us_name"];
        } else {
            // 担当者がいなかった場合の処理
            /* 全ユーザ検索は最初の1回だけ */
            if ($allUsers === "") {	/* foreachの外で定義された変数 */
                $s = $pdo->query(
                    "SELECT us_mail FROM user_tab WHERE us_active = true",
                    PDO::FETCH_ASSOC);

                foreach ($s as $x) {
                    /* 最初の1件はカンマをつけない */
                    $allUsers .= $allUsers ? ",".$x["us_mail"] : $x["us_mail"];
                }
                $s = NULL;

                // アクティブな人がいなかったらメール送らない
                if (empty($allUsers) || $allUsers == ',') {
                    $allUsers = false;
                }
            } elseif ($allUsers === false) {
                continue;
            }

            $to = $allUsers;
            $tags["user"] = " - ";
        }

        /* その他のメールテンプレート置換タグをセット */
        // latedaysはループ前にセット済み
        // contact_no, co_id, user はセット済み
        $tags["status"] = $row["st_status"];
        $tags["sender"] = $row["ma_from_addr"] ?: " - ";
        $tags["subject"] = $row["ma_subject"];
        // 日付はマイクロ秒まで取れてしまうので文字列を切り出す
        $tags["inquiry"] = substr($row["co_inquiry"], 0, 19) ?: " - ";
        $tags["lastupdate"] = substr($row["co_lastupdate"], 0, 19) ?: " - ";
        $tags["limit"] = substr($row["co_limit"], 0, 19) ?: " - ";
        $mailTmpl->assign($tags);
        /* 第二引数を指定するとフェッチモード */
        $rawTmplStr = $mailTmpl->display(MAIL_TMPL, true);

        /* メールテンプレート解析 */
        // 改行＋改行を探してヘッダ分割
        $mailItems = explode(PHP_EOL . PHP_EOL, $rawTmplStr, 2);
        if (!isset($mailItems[1])) {
            throw new CuMAS_Exception(MAIL_TMPL . "is the wrong format.");
        }

        // ヘッダ解析
        $headers = [
            "from" => "",
            "subject" => "",
            "option" => "",
        ];
        foreach (explode(PHP_EOL, $mailItems[0]) as $line) {
            $oneHeader = array_map('trim', explode(":", $line, 2));
            if (!isset($oneHeader[1])) {
                throw new CuMAS_Exception(
                    "Mail template has wrong header '{$oneHeader[0]}'");
            }

            switch (strtolower($oneHeader[0])) {
                case 'subject':
                    $headers["subject"] = $oneHeader[1];
                    break;

                case 'from':
                    $tmpFrom = explode('<', $oneHeader[1], 2);
                    /* <>を含まない形式だったらそのままアドレスとして使う */
                    if (!isset($tmpFrom[1])) {
                        $headers["from"] = $oneHeader[1];
                        break;
                    }

                    /* 前半をmimeエンコード */
                    $headers["from"] = mb_encode_mimeheader(
                        mb_convert_encoding(trim($tmpFrom[0]), "JIS", "UTF-8")
                    );

                    $headers["from"] .= " <${tmpFrom[1]}";
                    break;

                /* Toヘッダはテンプレートファイルには書かない */
                case 'to':
                    CuMAS_Exception::log_s($logFacility, __FILE__, "Mail template has wrong header '{$oneHeader[0]}'");
                    break;

                default:
                    /* 最後にFromをつけるので改行は全部OK */
                    $headers["option"] .= "{$line}\r\n";
                    break;
            }

        }	/* ヘッダ解析ここまで */

        // メール送信
        mb_send_mail($to,
                     $headers["subject"] ?: DEFAULT_SUBJECT,
                     $mailItems[1],		/* body */
                     $headers["option"] .  "From:" .
                         $headers["from"] ?: DEFAULT_MAIL_FROM);
    }

} catch (PDOException $e) {
    $ce = new CuMAS_Exception($e->getMessage());
    $ce->log($logFacility, __FILE__);
    exit(1);
} catch (CuMAS_Exception $ce) {
    $ce->log($logFacility, __FILE__);
    exit(1);
}


/* vim: set filetype=php: */
/* End of file cumascheck.php */
